#ifndef GRAPHICS_BITMAP_H
#define GRAPHICS_BITMAP_H TRUE

/*
**  bitmap.h
**
**  (C) Copyright 1996-2004 Rocklyte Systems.
**      All Rights Reserved.
*/

#ifndef MAIN_H
#include <pandora/main.h>
#endif

#ifdef __xwindows__
#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>
#undef NULL
#define NULL 0
#endif

#ifdef __snap__
#include <snap/graphics.h>
#endif

/****************************************************************************/

#define VER_PALETTE 1

struct RGBPalette {
   WORD ID;              /* ID_PALETTE */
   WORD Version;         /* Version */
   LONG AmtColours;      /* Amount of Colours */
   struct BRGB Col[256];  /* RGB Palette */
};

/****************************************************************************/

struct ColourFormat {
   UBYTE  RedShift;    /* Right shift value (applies only to 15/16 bit formats for eliminating redundant bits) */
   UBYTE  BlueShift;
   UBYTE  GreenShift;
   UBYTE  AlphaShift;
   UBYTE  RedMask;     /* The unshifted mask value (ranges from 0x00 to 0xff) */
   UBYTE  GreenMask;
   UBYTE  BlueMask;
   UBYTE  AlphaMask;
   UBYTE  RedPos;      /* Left shift/positional value */
   UBYTE  GreenPos;
   UBYTE  BluePos;
   UBYTE  AlphaPos;
};

/*****************************************************************************
** Bitmap class.
*/

#define VER_BITMAP 1.0

struct Bitmap {
   OBJECT_HEADER               /* Class header */
   struct RGBPalette *Palette; /* Pointer to the Bitmap's palette */
   struct RGB *ColourRGB;      /* Background/transparent colour */
   UBYTE  *Data;               /* Pointer to bitmap data area */
   LONG   Width;               /* Width */
   LONG   ByteWidth;           /* ByteWidth (not including padding - see LineWidth) */
   LONG   Height;              /* Height */
   LONG   Type;                /* Bitmap type */
   LONG   LineWidth;           /* Line differential in bytes */
   LONG   PlaneMod;            /* Plane differential in bytes */
   LONG   ClipLeft;            /* Left side clipping */
   LONG   ClipRight;           /* Right side clipping */
   LONG   ClipBottom;          /* Bottom side clipping */
   LONG   ClipTop;             /* Top side clipping */
   LONG   Size;                /* Total size of the bitmap in bytes */
   LONG   DataFlags;           /* Memory flags to use in allocation */
   LONG   AmtColours;          /* Maximum amount of colours available */
   LONG   Flags;               /* Optional flags */
   LONG   ColourIndex;         /* Colour index or packed colour value that acts as the background/transparency */
   LONG   BytesPerPixel;       /* 1, 2, 3, 4 */
   LONG   BitsPerPixel;        /* 1, 8, 15, 16, 24, 32 */
   LONG   Position;            /* Current byte position for reading and writing */
   LONG   XOffset;             /* Horizontal offset to apply to graphics coordinates */
   LONG   YOffset;             /* Vertical offset to apply to graphics coordinates */
   LONG   Opacity;             /* Opacity setting to use in drawing operations */
   void   (*DrawUCPixel)(struct Bitmap *,  LONG, LONG, ULONG);
   void   (*DrawUCRPixel)(struct Bitmap *, LONG, LONG, struct RGB *);
   ULONG  (*ReadUCPixel)(struct Bitmap *,  LONG, LONG);
   void   (*ReadUCRPixel)(struct Bitmap *, LONG, LONG, struct RGB *);
   struct ColourFormat *ColourFormat;
   void   (*ReadUCRIndex)(struct Bitmap *, UBYTE *, struct BRGB *);
   void   (*DrawUCRIndex)(struct Bitmap *, UBYTE *, struct BRGB *);
   void   (*DrawUCIndex)(struct Bitmap *, UBYTE *, ULONG);
   MEMORYID DataMID;           /* Data memory ID */

   PRIVATE_FIELDS
   struct RGBPalette prvPaletteArray;
   struct RGB prvColourRGB;           /* Background colour */
   LONG   prvAFlags;                  /* Private allocation flags */
   struct ColourFormat prvColourFormat;

#ifdef __xwindows__
   XImage   prvXImage;
   Drawable prvDrawable;
   XImage   *prvReadable;
   XShmSegmentInfo prvShmInfo;
   BYTE prvXShmImage;
#endif

#ifdef _WIN32
   APTR prvDrawable;    /* HDC for the Bitmap */
#endif

#ifdef __snap__
   GA_buf *prvBuffer;
   WORD prvBufferLock;
#endif
};

/*** Standard pack pixel for all formats ***/

#define PackPixel(a,b,c,d) ((((b)>>(a)->ColourFormat->RedShift) & (a)->ColourFormat->RedMask) << (a)->ColourFormat->RedPos) | ((((c)>>(a)->ColourFormat->GreenShift) & (a)->ColourFormat->GreenMask) << (a)->ColourFormat->GreenPos) | ((((d)>>(a)->ColourFormat->BlueShift) & (a)->ColourFormat->BlueMask) << (a)->ColourFormat->BluePos)
#define PackPixelA(a,b,c,d,e) ((((b)>>(a)->ColourFormat->RedShift) & (a)->ColourFormat->RedMask) << (a)->ColourFormat->RedPos) | \
                              ((((c)>>(a)->ColourFormat->GreenShift) & (a)->ColourFormat->GreenMask) << (a)->ColourFormat->GreenPos) | \
                              ((((d)>>(a)->ColourFormat->BlueShift) & (a)->ColourFormat->BlueMask) << (a)->ColourFormat->BluePos) | \
                              ((((e)>>(a)->ColourFormat->AlphaShift) & (a)->ColourFormat->AlphaMask) << (a)->ColourFormat->AlphaPos)
#define PackAlpha(a,b) ((((b)>>(a)->ColourFormat->AlphaShift) & (a)->ColourFormat->AlphaMask) << (a)->ColourFormat->AlphaPos)

/*** Pack pixel 'whole-byte' version, for faster 24/32 bit formats ***/

#define PackPixelWB(a,b,c,d) ((b) << (a)->ColourFormat->RedPos) | ((c) << (a)->ColourFormat->GreenPos) | ((d) << (a)->ColourFormat->BluePos)
#define PackPixelWBA(a,b,c,d,e) ((b) << (a)->ColourFormat->RedPos) | ((c) << (a)->ColourFormat->GreenPos) | ((d) << (a)->ColourFormat->BluePos) | ((e) << (a)->ColourFormat->AlphaPos)

/*** Colour unpacking routines ***/

#define UnpackRed(a,b)   ((((b) >> (a)->ColourFormat->RedPos) & (a)->ColourFormat->RedMask) << (a)->ColourFormat->RedShift)
#define UnpackGreen(a,b) ((((b) >> (a)->ColourFormat->GreenPos) & (a)->ColourFormat->GreenMask) << (a)->ColourFormat->GreenShift)
#define UnpackBlue(a,b)  ((((b) >> (a)->ColourFormat->BluePos) & (a)->ColourFormat->BlueMask) << (a)->ColourFormat->BlueShift)
#define UnpackAlpha(a,b) ((((b) >> (a)->ColourFormat->AlphaPos) & (a)->ColourFormat->AlphaMask))

struct Rectangle {
   union { LONG x; LONG XCoord; };
   union { LONG y; LONG YCoord; };
   union { LONG width; LONG Width; };
   union { LONG height; LONG Height; };
};

struct ClipRectangle {
   union { LONG left; LONG Left; };
   union { LONG top; LONG Top; };
   union { LONG right; LONG Right; };
   union { LONG bottom; LONG Bottom; };
};

/*****************************************************************************
** Bitmap Types
*/

#define BMP_ILBM   1
#define BMP_PLANAR 2
#define BMP_CHUNKY 3
#define BMP_INTERLEAVED ILBM

/*****************************************************************************
** Bitmap Flags
*/

#define BMF_BLANKPALETTE  0x00000001  /* For a blank/black palette */
#define BMF_NODATA        0x00000008  /* Do not allocate Bitmap->Data */
#define BMF_TRANSPARENT   0x00000010  /* Automatically manage optimum bitmap transparency */
#define BMF_CLIPPING      0x00000020  /* Clipping is on */
#define BMF_MASK          0x00000040  /* Declare the Bitmap as a 1 or 8 bit mask */
#define BMF_INVERSEALPHA  0x00000080  /* Indicates reverse alpha blending, higher values are transparent */
#define BMF_CLEAR         0x00000200  /* Clear graphics on initialisation and when resizing */
#define BMF_USER          0x00000400  /* This user flag is to allow tagging of bitmaps for special meaning, it is not used internally */
#define BMF_2DACCELERATED 0x00000800  /* 2D video acceleration is available */
#define BMF_3DACCELERATED 0x00001000  /* 3D video acceleration is available */
#define BMF_ALPHABLEND    0x00002000  /* For 32-bit images, this flag can be set to enable alpha blending when performing a blit.  Can be set either in the source or destination */
#define BMF_ALPHA         0x00004000  /* For 32-bit images, indicates that an alpha channel is present */
#define BMF_NEVERSHRINK   0x00008000  /* Ignore resize requests that would shrink the size of the bitmap */
#define BMF_X11DGA        0x00010000  /* Private DGA indicator */
#define BMF_FIXEDDEPTH    0x00020000  /* Do not allow anyone to change the bitmap depth after initialisation */

/*****************************************************************************
** Bitmap PixelMode Settings
*/

#define PIXEL_READ   1
#define PIXEL_WRITE  2
#define PIXEL_UPDATE 3

/*****************************************************************************
** Bitmap Methods
*/

#define MT_BlitArea         -1
#define MT_ReadPixel        -2
#define MT_ReadRGBPixel     -3
#define MT_DrawPixel        -4
#define MT_DrawRGBLine      -5
#define MT_DrawRGBPixel     -6
#define MT_DrawRGBRectangle -7
#define MT_FlipBitmap       -8
#define MT_FloodRGB         -9
#define MT_DrawRectangle   -10
#define MT_SetClipRegion   -11
#define MT_WaitBlit        -12
#define MT_GetColour       -13
#define MT_DrawLine        -14
#define MT_BlitStretch     -15

#define FLIP_HORIZONTAL 1
#define FLIP_VERTICAL   2

#define BSF_BILINEAR     0x00000001
#define BSF_FILTERSOURCE 0x00000002
#define BSF_GOODQUALITY  BSF_BILINEAR

#define SetDrawPixel(a,b,c,d)    a.XCoord=(b); a.YCoord=(c); a.Colour=(d);
#define SetDrawLine(a,b,c,d,e,f) a.XCoord=(b); a.YCoord=(c); a.XEnd=(d); a.YEnd=(e); a.Red=(f->Red); a.Green=(f->Green); a.Blue=(f->Blue);
#define SetDrawRGBPixel(a,b,c,d) a.XCoord=(b); a.YCoord=(c); a.Red=(d->Red); a.Green=(d->Green); a.Blue=(d->Blue);
#define SetDrawRectangle(a,b,c,d,e,f,g) a.XCoord=(b); a.YCoord=(c); a.Width=(d); a.Height=(e); a.RGB=(f); a.Fill=(g);

struct mtBlitArea         { struct Bitmap *DestBitmap; LONG Flags; LONG XCoord; LONG YCoord; LONG Width; LONG Height; LONG XDest; LONG YDest; };
struct mtBlitStretch      { struct Bitmap *DestBitmap; LONG Flags; LONG XCoord; LONG YCoord; LONG Width; LONG Height; LONG XDest; LONG YDest; LONG DestWidth; LONG DestHeight; };
struct mtClipRegion       { LONG Number; LONG Left; LONG Top; LONG Right; LONG Bottom; LONG Terminate; };
struct mtDrawLine         { LONG XCoord; LONG YCoord; LONG XEnd; LONG YEnd; LONG Colour; };
struct mtDrawPixel        { LONG XCoord; LONG YCoord; ULONG Colour; };
struct mtDrawRGBLine      { LONG XCoord; LONG YCoord; LONG XEnd; LONG YEnd; LONG Red; LONG Green; LONG Blue; };
struct mtDrawRGBPixel     { LONG XCoord; LONG YCoord; LONG Red; LONG Green; LONG Blue; };
struct mtDrawRGBRectangle { LONG XCoord; LONG YCoord; LONG Width; LONG Height; LONG Red; LONG Green; LONG Blue; LONG Fill; };
struct mtDrawRectangle    { LONG XCoord; LONG YCoord; LONG Width; LONG Height; LONG Colour; LONG Fill; };
struct mtFlipBitmap       { LONG Orientation; };
struct mtFloodRGB         { LONG XCoord; LONG YCoord; LONG Red; LONG Green; LONG Blue; };
struct mtFlood            { LONG XCoord; LONG YCoord; LONG Colour; };
struct mtReadPixel        { LONG XCoord; LONG YCoord; LONG Colour; };
struct mtReadRGBPixel     { LONG XCoord; LONG YCoord; LONG Red; LONG Green; LONG Blue; };
struct mtSetClipRegion    { LONG Number; LONG Left; LONG Top; LONG Right; LONG Bottom; LONG Terminate; };
struct mtGetColour        { LONG Red; LONG Green; LONG Blue; LONG Alpha; LONG Colour; };

#define mtDrawRectangle(bmp,x,y,w,h,c,f)        (ActionTags(MT_DrawRectangle,(bmp),(x),(y),(w),(h),(c),(f)))
#define mtDrawRGBRectangle(bmp,x,y,w,h,r,g,b,f) (ActionTags(MT_DrawRGBRectangle,(bmp),(x),(y),(w),(h),(r),(g),(b),(f)))

/*****************************************************************************
** Region class.
*/

#define VER_REGION 1.0

struct RegionRect {
   LONG x1, y1, x2, y2;
};

struct BitmapRegion {
   OBJECT_HEADER
   LONG Size;
   LONG Total;
   struct RegionRect Overall;  /* Overall X/Y coordinates */
   struct RegionRect *Array;
};

enum {
   RECT_INSIDE,    /* Rectangle is in region */
   RECT_OUTSIDE,   /* Rectangle not in region */
   RECT_PARTIAL    /* Rectangle partially in region */
};

/*****************************************************************************/

#define MT_SetRegion       -1
#define MT_TranslateRegion -2
#define MT_PointInRegion   -3
#define MT_XORRegion       -4
#define MT_CompareRegions  -5
#define MT_RegionUnion     -6
#define MT_ShrinkRegion    -7
#define MT_IntersectRegion -8
#define MT_AddRegionRect   -9
#define MT_RectInRegion    -10
#define MT_SubtractRegion  -11
#define MT_AddRegion       -12

struct mtAddRegion {
   struct BitmapRegion *Region;
};

struct mtAddRegionRect {
   LONG XCoord;
   LONG YCoord;
   LONG Width;
   LONG Height;
};

struct mtIntersectRegion {
   struct BitmapRegion *Region;
};

struct mtShrinkRegion {
   LONG XCoord;
   LONG YCoord;
};

struct mtRegionUnion {
   struct BitmapRegion *Region;
};

struct mtCompareRegions {
   struct BitmapRegion *Region;
};

struct mtSetRegion {
   LONG XCoord;
   LONG YCoord;
   LONG Width;
   LONG Height;
};

struct mtSubtractRegion {
   struct BitmapRegion *Region;
};

struct mtTranslateRegion {
   LONG XCoord;
   LONG YCoord;
};

struct mtPointInRegion {
   LONG XCoord;
   LONG YCoord;
};

struct mtRectInRegion {
   LONG XCoord;
   LONG YCoord;
   LONG Width;
   LONG Height;
   LONG Result;
};

struct mtXORRegion {
   APTR Region;
};

/*****************************************************************************
** Pen class.
*/

#define VER_PEN 1.0

struct BitmapPen {
   OBJECT_HEADER
   struct Bitmap *Bitmap;    /* Bitmap to draw to */
   LONG TipShape;            /* Square, Circle, Pixel */
   LONG PenShape;            /* Square, Circle, Pixel, Rectangle, Line, Ellipse */
   LONG TipSize;             /* Radius or width of the pen tip */
   LONG XCoord;
   LONG YCoord;
   LONG Width;
   LONG Height;
   LONG XEnd;
   LONG YEnd;
   LONG Fill;
   LONG Red;                 /* Foreground Red */
   LONG Green;               /* Foreground Green */
   LONG Blue;                /* Foreground Blue */
   LONG Alpha;               /* Alpha blending to apply */
   LONG Pattern;             /* Special bit pattern */
   LONG Operation;           /* Operation to apply */
   LONG BRed;                /* Background Red */
   LONG BGreen;              /* Background Green */
   LONG BBlue;               /* Background Blue */
   struct Bitmap *Source;    /* Source bitmap, if copying/tiling */

   PRIVATE_FIELDS
   void (*DrawPen)(struct BitmapPen *, LONG X, LONG Y);
};

#define POP_COPY    0x00000000
#define POP_XOR     0x00000001
#define POP_INVERSE 0x00000002

/*****************************************************************************
** Pen Tips
*/

#define TIP_CIRCLE 1
#define TIP_SQUARE 2
#define TIP_PIXEL  3

/*****************************************************************************
** Pen Shapes
*/

#define PEN_LINE      1
#define PEN_SQUARE    2
#define PEN_POINT     3
#define PEN_RECTANGLE 4
#define PEN_ELLIPSE   5
#define PEN_ARC       6

/*****************************************************************************
** Pen Methods
*/

#define MT_PenEllipse      -2
#define MT_PenLine         -3
#define MT_PenPoint        -4
#define MT_PenRectangle    -5
#define MT_PenArc          -6

struct mtPenArc {
   LONG XCoord;
   LONG YCoord;
   LONG Radius;
   LONG Angle1;
   LONG Angle2;
};

struct mtPenEllipse {
   LONG XCoord;
   LONG YCoord;
   LONG Width;
   LONG Height;
};

struct mtPenLine {
   LONG XCoord;
   LONG YCoord;
   LONG XEnd;
   LONG YEnd;
};

struct mtPenPoint {
   LONG XCoord;
   LONG YCoord;
};

struct mtPenRectangle {
   LONG XCoord;
   LONG YCoord;
   LONG Width;
   LONG Height;
};

#endif /* GRAPHICS_BITMAP_H */

