#ifndef SYSTEM_TASK_H
#define SYSTEM_TASK_H TRUE

/*
**  task.h
**
**  (C) Copyright 1996-2004 Rocklyte Systems.
**      All Rights Reserved.
**
*/

#define MAX_TASKS    50  /* Maximum number of tasks allowed to run at once */
#define MAX_MEMLOCKS 64  /* Maximum number of non-blocking memory locks allowed per task */
#define MAX_SEMLOCKS 40  /* Maximum number of semaphore allocations per task */

/*****************************************************************************
** Task flags
*/

#define TSF_FOREIGN    0x00000001
#define TSF_DUMMY      0x00000002
#define TSF_WAIT       0x00000004
#define TSF_RESETDIR   0x00000008
#define TSF_PRIVILEGED 0x00000010

/*****************************************************************************
** Task methods
*/

#define MT_ExpungeTask     -1
#define MT_AddTaskArgument -2
#define MT_QuitTask        -3
#define MT_Shutdown        -4
#define MT_Reboot          -5
#define MT_CloseInstance   -6

struct mtAddTaskArgument {
   STRING Argument;
};

/*****************************************************************************
** Private task list control structure.
*/

struct TaskList {
   LONG     ProcessID;         /* Kernel process ID */
   OBJECTID TaskID;            /* Task ID for this array entry */
   LONG     WaitingProcessID;  /* If the task is waiting, this field reflects the other task's ID */
   MEMORYID WaitingMemoryID;   /* If the task is waiting, this field reflects the memory ID */
   LONG     WaitingTime;       /* If the task is waiting, the time at which the sleep started (msec) */
   MEMORYID MessageID;         /* Message queue ID */
   OBJECTID OutputID;          /* The object that the task should output information to */
   LONG     Semaphore;         /* The semaphore to signal when waking up the task (win32) */
   LONG     InstanceID;        /* Instance that the task belongs to */
   WORD     LastAction;        /* ID of the last action/method that was executed */
   WORD     Index;             /* Index in the shTasks array */
   struct   {
      MEMORYID MemoryID;
      WORD     AccessCount;
   } MemoryLocks[MAX_MEMLOCKS+1]; /* Allow for a NULL entry at the end of the array */
};

#endif /* SYSTEM_TASK_H */

